@@ -10,8 +10,9 @@
 #' @param model Model object
 #' @param newdata data.frame (one and only one of the `model` and `newdata` arguments can be used.)
 #' @param by character vector with grouping variables within which `FUN_*` functions are applied to create "sub-grids" with unspecified variables.
+#' @param response Logical should the response variable be included in the grid, even if it is not specified explicitly.
 #' @param FUN_character the function to be applied to character variables.
-#' @param FUN_factor the function to be applied to factor variables.
+#' @param FUN_factor the function to be applied to factor variables. This only applies if the variable in the original data is a factor. For variables converted to factor in a model-fitting formula, for example, `FUN_character` is used.
 #' @param FUN_logical the function to be applied to logical variables.
 #' @param FUN_integer the function to be applied to integer variables.
 #' @param FUN_binary the function to be applied to binary variables.
@@ -75,6 +76,7 @@ datagrid <- function(
     newdata = NULL,
     by = NULL,
     grid_type = "mean_or_mode",
+    response = FALSE,
     FUN_character = NULL,
     FUN_factor = NULL,
     FUN_logical = NULL,
@@ -101,6 +103,9 @@ datagrid <- function(
     checkmate::assert_function(FUN_other, null.ok = TRUE)
     checkmate::assert_character(by, null.ok = TRUE)
     checkmate::assert_data_frame(newdata, null.ok = TRUE)
+    checkmate::assert_flag(response)
+
+    explicit <- c(names(dots), by)
 
     if (grid_type == "mean_or_mode") {
         if (is.null(FUN_character)) FUN_character <- get_mode
@@ -114,7 +119,8 @@ datagrid <- function(
     } else if (grid_type == "balanced") {
         if (is.null(FUN_character)) FUN_character <- unique
         if (is.null(FUN_logical)) FUN_logical <- unique
-        if (is.null(FUN_factor)) FUN_factor <- unique
+        # not just levels(), because that is string, and sorts badly ex: "2" vs "10"
+        if (is.null(FUN_factor)) FUN_factor <- function(k) sort(unique(k))
         if (is.null(FUN_binary)) FUN_binary <- unique
         if (is.null(FUN_numeric)) FUN_numeric <- function(x) mean(x, na.rm = TRUE)
         if (is.null(FUN_other)) FUN_other <- function(x) mean(x, na.rm = TRUE)
@@ -149,6 +155,7 @@ datagrid <- function(
             args <- c(list(...), list(
                 model = model,
                 newdata = newdata_list[[i]],
+                response = response,
                 FUN_character = FUN_character,
                 FUN_factor = FUN_factor,
                 FUN_logical = FUN_logical,
@@ -162,14 +169,19 @@ datagrid <- function(
             }
             newdata_list[[i]] <- do.call(datagrid_engine, args)
         }
+
         out <- data.table::rbindlist(newdata_list)
         data.table::setDF(out)
+
+        attr(out, "explicit") <- explicit
+
         return(out)
     }
     
     out <- datagrid_engine(...,
                 model = model,
                 newdata = newdata,
+                response = response,
                 FUN_character = FUN_character,
                 FUN_factor = FUN_factor,
                 FUN_logical = FUN_logical,
@@ -177,14 +189,21 @@ datagrid <- function(
                 FUN_numeric = FUN_numeric,
                 FUN_integer = FUN_integer,
                 FUN_other = FUN_other)
+
+    if (!"rowid" %in% colnames(out)) {
+        out$rowid <- seq_len(nrow(out))
+    }
+
+    attr(out, "explicit") <- explicit
     return(out)
 }
 
-        
+
 datagrid_engine <- function(
     ...,
     model = NULL,
     newdata = NULL,
+    response = response,
     FUN_character = get_mode,
     # need to be explicit for numeric variables transfered to factor in model formula
     FUN_factor = get_mode,
@@ -204,14 +223,27 @@ datagrid_engine <- function(
     variables_all <- tmp$all
     variables_manual <- names(at)
     variables_automatic <- tmp$automatic
-    
-    # commented out because we want to keep the response in
+
+    # usually we don't want the response in the grid, but 
     # sometimes there are two responses and we need one of them:
     # brms::brm(y | trials(n) ~ x + w + z)
-    # if (!is.null(model)) {
-    #     variables_automatic <- setdiff(variables_automatic, insight::find_response(model))
-    # }
+    if (!is.null(model) && isFALSE(response)) {
+        resp <- insight::find_response(model)
+        if (inherits(model, "brmsfit")) {
+            fl <- as.character(stats::formula(model))
+            matches <- regexpr("trials\\(.*?\\)", fl)
+            extracted <- regmatches(fl, matches)[1]
+            if (isFALSE(is.na(extracted[1]))) {
+                extracted <- gsub("trials\\((.*)\\)", "\\1", extracted)
+            } else {
+                extracted <- NULL
+            }
+            extracted <- unlist(extracted)
+            resp <- setdiff(resp, extracted)
+        }
 
+        variables_automatic <- setdiff(variables_automatic, resp)
+    }
 
     if (length(variables_automatic) > 0) {
         idx <- intersect(variables_automatic, colnames(dat))
@@ -222,7 +254,11 @@ datagrid_engine <- function(
         # created by insight::get_data
         for (n in names(dat_automatic)) {
             if (get_variable_class(dat, n, c("factor", "strata", "cluster")) || n %in% tmp[["cluster"]]) {
-                out[[n]] <- FUN_factor(dat_automatic[[n]])
+                if (is.factor(dat_automatic[[n]])) {
+                    out[[n]] <- FUN_factor(dat_automatic[[n]])
+                } else {
+                    out[[n]] <- FUN_character(dat_automatic[[n]])
+                }
             } else if (get_variable_class(dat, n, "binary")) {
                 out[[n]] <- FUN_binary(dat_automatic[[n]])
             } else if (get_variable_class(dat, n, "logical")) {
@@ -243,6 +279,7 @@ datagrid_engine <- function(
         out <- list()
     }
 
+
     if (!is.null(at)) {
         for (n in names(at)) {
             out[n] <- at[n]
@@ -253,7 +290,7 @@ datagrid_engine <- function(
     out <- lapply(out, unique)
 
     # warn on very large prediction grid
-    num <- as.numeric(sapply(out, length)) # avoid integer overflow
+    num <- as.numeric(lengths(out)) # avoid integer overflow
     num <- Reduce(f = "*", num)
     if (isTRUE(num > 1e9)) {
         stop("You are trying to create a prediction grid with more than 1 billion rows, which is likely to exceed the memory and computational power available on your local machine. Presumably this is because you are considering many variables with many levels. All of the functions in the `marginaleffects` package include arguments to specify a restricted list of variables over which to create a prediction grid.", call. = FALSE)
@@ -263,17 +300,9 @@ datagrid_engine <- function(
     args <- c(out, list(sorted = FALSE))
     out <- do.call("fun", args)
 
-    # na.omit destroys attributes, and we need the "factor" attribute
-    # created by insight::get_data
-    for (n in names(out)) {
-        attr(out, "marginaleffects_variable_class") <- attr(dat, "marginaleffects_variable_class")
-    }
-
     # better to assume "standard" class as output
     data.table::setDF(out)
 
-    attr(out, "variables_datagrid") <- names(dots)
-
     return(out)
 }
 
@@ -308,7 +337,7 @@ datagridcf_internal <- function(
     if (length(variables_automatic) > 0) {
         idx <- intersect(variables_automatic, colnames(dat))
         dat_automatic <- dat[, ..idx, drop = FALSE]
-        dat_automatic[, rowidcf := rowid$rowidcf]
+        dat_automatic$rowidcf <- rowid$rowidcf
         setcolorder(dat_automatic, c("rowidcf", setdiff(names(dat_automatic), "rowidcf")))
         # cross-join 2 data.tables, faster than merging two dataframes
         out <- cjdt(list(dat_automatic, at))
@@ -350,16 +379,17 @@ prep_datagrid <- function(..., model = NULL, newdata = NULL, by = NULL) {
         insight::format_error(msg)
     }
 
-    if (!is.null(model)) {
-        variables_list <- insight::find_variables(model)
-        variables_all <- unlist(variables_list, recursive = TRUE)
-        # weights are not extracted by default
-        variables_all <- c(variables_all, insight::find_weights(model))
-
-    } else if (!is.null(newdata)) {
+    # newdata before model: if user supplies newdata explicitly, they might want 
+    # all columns for something like `hypothesis = function()`
+    if (!is.null(newdata)) {
         variables_list <- NULL
         variables_all <- colnames(newdata)
         newdata <- set_variable_class(modeldata = newdata, model = model)
+    } else if (!is.null(model)) {
+        variables_list <- insight::find_variables(model, verbose = FALSE)
+        variables_all <- unlist(variables_list, recursive = TRUE)
+        # weights are not extracted by default
+        variables_all <- c(variables_all, insight::find_weights(model))
     }
 
     variables_manual <- names(at)
@@ -369,13 +399,13 @@ prep_datagrid <- function(..., model = NULL, newdata = NULL, by = NULL) {
     if (is.null(newdata)) {
         newdata <- get_modeldata(model, additional_variables = FALSE)
     }
-    
+
     attr_variable_classes <- attr(newdata, "marginaleffects_variable_class")
 
-    # subset columns, otherwise it can be ultra expensive to compute summaries for every variable
-    if (!is.null(model)) {
+    # subset columns, otherwise it can be ultra expensive to compute summaries for every variable. But do the expensive thing anyway if `newdata` is supplied explicitly by the user, or in counterfactual grids.
+    if (!is.null(model) && is.null(newdata)) {
         variables_sub <- c(
-            hush(insight::find_variables(model, flatten = TRUE)),
+            hush(insight::find_variables(model, flatten = TRUE, verbose = FALSE)),
             hush(unlist(insight::find_weights(model), use.names = FALSE))) # glmmTMB needs weights column for predictions
         variables_sub <- c(variables_sub, variables_manual)
         variables_sub <- c(variables_sub, c("marginaleffects_wts_internal", "rowid_dedup"))
@@ -396,7 +426,7 @@ prep_datagrid <- function(..., model = NULL, newdata = NULL, by = NULL) {
     idx <- vapply(newdata, is.matrix, logical(1L))
     if (any(idx)) {
         if (any(names(newdata)[idx] %in% variables_all)) {
-            insight::format_warning("Matrix columns are not supported as predictors and are therefore omitted. This may prevent computation of the quantities of interest. You can construct your own prediction dataset and supply it explicitly to the `newdata` argument.")
+            insight::format_warning("Matrix columns are not supported as predictors and are therefore omitted. This may prevent computation of the quantities of interest. You can construct your own prediction dataset and supply it explicitly to the `newdata` argument. This issue can happen when using functions like `scale()`, which returns matrices rather than vectors. Convert your column to a vector before fitting you model.")
         }
         newdata <- newdata[, !idx, drop = FALSE]
     }
@@ -435,7 +465,7 @@ prep_datagrid <- function(..., model = NULL, newdata = NULL, by = NULL) {
     
     # cluster identifiers will eventually be treated as factors
     if (!is.null(model)) {
-        v <- insight::find_variables(model)
+        v <- insight::find_variables(model, verbose = FALSE)
         v <- unlist(v[names(v) %in% c("cluster", "strata")], recursive = TRUE)
         variables_cluster <- c(v, insight::find_random(model, flatten = TRUE))
     } else {
