@@ -5,8 +5,7 @@ get_contrasts <- function(model,
                           original,
                           lo,
                           hi,
-                          wts = NULL,
-                          marginalmeans,
+                          wts = FALSE,
                           by = NULL,
                           hypothesis = NULL,
                           cross = FALSE,
@@ -87,6 +86,7 @@ get_contrasts <- function(model,
         } else {
             pred_hi <- pred_hi$error
         }
+
     }
 
     # predict() takes up 2/3 of the wall time. This call is only useful when we
@@ -119,7 +119,10 @@ get_contrasts <- function(model,
     # lots of indexing later requires a data.table
     data.table::setDT(original)
 
-    if (!inherits(pred_hi, "data.frame") || !inherits(pred_lo, "data.frame") || !inherits(pred_or, c("data.frame", "NULL"))) {
+    if (!inherits(pred_hi, "data.frame") || 
+        !inherits(pred_lo, "data.frame") || 
+        !inherits(pred_or, c("data.frame", "NULL")) ||
+        all(is.na(pred_lo$estimate))) {
         msg <- "Unable to compute predicted values with this model. This error can arise when `insight::get_data()` is unable to extract the dataset from the model object, or when the data frame was modified since fitting the model. You can try to supply a different dataset to the `newdata` argument."
         if (inherits(pred_hi, c("try-error", "error"))) {
             msg <-c(msg, "", "In addition, this error message was raised:", "", as.character(pred_hi)) 
@@ -175,9 +178,11 @@ get_contrasts <- function(model,
 
         if (length(tmp) == 0) {
             if (all(colnames(by) %in% c("by", colnames(newdata)))) {
-                nd <- c("rowid", setdiff(colnames(by), "by"))
-                nd <- newdata[, nd, drop = FALSE]
-                out <- merge(out, nd, by = "rowid", sort = FALSE)
+                nd <- c("rowid", "rowid_dedup", setdiff(colnames(by), "by"))
+                nd <- intersect(nd, colnames(newdata))
+                nd <- newdata[, ..nd, drop = FALSE]
+                bycol <- intersect(c("rowid", "rowid_dedup"), colnames(nd))
+                out <- merge(out, nd, by = bycol, sort = FALSE)
                 tmp <- setdiff(intersect(colnames(out), colnames(by)), "by")
             } else {
                 insight::format_error("The column in `by` must be present in `newdata`.")
@@ -195,6 +200,11 @@ get_contrasts <- function(model,
         out[by, by := by, on = tmp]
         by <- "by"
 
+    } else if (isTRUE(by)) {
+        regex <- "^term$|^contrast_?|^group$"
+        by <- grep(regex, colnames(out), value = TRUE)
+        by <- unique(by)
+
     } else if (isTRUE(checkmate::check_character(by))) {
         regex <- "^term$|^contrast_?|^group$"
         by <- c(by, grep(regex, colnames(out), value = TRUE))
@@ -296,15 +306,6 @@ get_contrasts <- function(model,
     # we feed these columns to safefun(), even if they are useless for categoricals
     if (!"marginaleffects_wts_internal" %in% colnames(out))  out[, "marginaleffects_wts_internal" := NA]
 
-    if (isTRUE(marginalmeans)) {
-        out <- out[, .(
-            predicted_lo = mean(predicted_lo),
-            predicted_hi = mean(predicted_hi),
-            predicted = mean(predicted),
-            marginaleffects_wts_internal = mean(marginaleffects_wts_internal)),
-        keyby = idx]
-    }
-
     # safe version of comparison
     # unknown arguments
     # singleton vs vector
@@ -365,7 +366,7 @@ get_contrasts <- function(model,
         draws <- draws[idx_na, , drop = FALSE]
 
         if (isTRUE(checkmate::check_character(by, min.len = 1))) {
-            by_idx <- subset(out, select = intersect(by, colnames(out)))
+            by_idx <- out[, ..by]
             by_idx <- do.call(paste, c(by_idx, sep = "|"))
         } else {
             by_idx <- out$term
@@ -395,7 +396,7 @@ get_contrasts <- function(model,
         # if comparison returns a single value, then we padded with NA. That
         # also means we don't want `rowid` otherwise we will merge and have
         # useless duplicates.
-        if (any(!idx)) {
+        if (!all(idx)) {
             if (settings_equal("marginaleffects_safefun_return1", TRUE)) {
                 out[, "rowid" := NULL]
             }
@@ -476,7 +477,7 @@ get_contrasts <- function(model,
     attr(out, "posterior_draws") <- draws
 
     # hypothesis tests using the delta method
-    out <- get_hypothesis(out, hypothesis, by = by)
+    out <- get_hypothesis(out, hypothesis, by = by, newdata = original, draws = draws)
 
     # reset settings
     settings_rm("marginaleffects_safefun_return1")
