@@ -9,7 +9,7 @@ get_predict.MCMCglmm <- function(
 
     ndraws_mod <- nrow(model$VCV)
     if (ndraws < ndraws_mod) {
-        idx <- sample(seq_len(ndraws_mod), ndraws)
+        idx <- sample.int(ndraws_mod, ndraws)
     } else {
         idx <- seq_len(ndraws_mod)
     }
@@ -32,4 +32,4 @@ get_vcov.MCMCglmm <- function(model,
         insight::format_warning("The `vcov` argument is not supported for models of this class.")
     }
     return(NULL)
-}
\ No newline at end of file
+}
